@@ -8,10 +8,8 @@ from unittest.mock import MagicMock
 
 import pytest
 
-import aiohttp.web
-from aiohttp.test_utils import make_mocked_request
-from aiohttp.web import HTTPCreated, Response
-from aiohttp.web_urldispatcher import PlainRoute, SystemRoute, UrlDispatcher
+from aiohttp import abc, web
+from aiohttp.web_urldispatcher import SystemRoute
 
 
 @pytest.fixture(scope='function')
@@ -40,7 +38,7 @@ def tmp_dir_path(request):
                            b'<li><a href="/my_file">my_file</a></li>\n'
                            b'</ul>\n</body>\n</html>')])
 @asyncio.coroutine
-def test_access_root_of_static_handler(tmp_dir_path, create_app_and_client,
+def test_access_root_of_static_handler(tmp_dir_path, loop, test_client,
                                        show_index, status, data):
     """
     Tests the operation of static file server.
@@ -60,10 +58,11 @@ def test_access_root_of_static_handler(tmp_dir_path, create_app_and_client,
     with open(my_file_path, 'w') as fw:
         fw.write('world')
 
-    app, client = yield from create_app_and_client()
+    app = web.Application(loop=loop)
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=show_index)
+    client = yield from test_client(app)
 
     # Request the root of the static directory.
     r = yield from client.get('/')
@@ -76,12 +75,42 @@ def test_access_root_of_static_handler(tmp_dir_path, create_app_and_client,
     yield from r.release()
 
 
+@pytest.mark.parametrize('data', ['hello world'])
+@asyncio.coroutine
+def test_follow_symlink(tmp_dir_path, loop, test_client, data):
+    """
+    Tests the access to a symlink, in static folder
+    """
+    my_dir_path = os.path.join(tmp_dir_path, 'my_dir')
+    os.mkdir(my_dir_path)
+
+    my_file_path = os.path.join(my_dir_path, 'my_file_in_dir')
+    with open(my_file_path, 'w') as fw:
+        fw.write(data)
+
+    my_symlink_path = os.path.join(tmp_dir_path, 'my_symlink')
+    os.symlink(my_dir_path, my_symlink_path)
+
+    app = web.Application(loop=loop)
+
+    # Register global static route:
+    app.router.add_static('/', tmp_dir_path, follow_symlinks=True)
+    client = yield from test_client(app)
+
+    # Request the root of the static directory.
+    r = yield from client.get('/my_symlink/my_file_in_dir')
+    assert r.status == 200
+    assert (yield from r.text()) == data
+
+    yield from r.release()
+
+
 @pytest.mark.parametrize('dir_name,filename,data', [
     ('', 'test file.txt', 'test text'),
     ('test dir name', 'test dir file .txt', 'test text file folder')
 ])
 @asyncio.coroutine
-def test_access_to_the_file_with_spaces(tmp_dir_path, create_app_and_client,
+def test_access_to_the_file_with_spaces(tmp_dir_path, loop, test_client,
                                         dir_name, filename, data):
     """
     Checks operation of static files with spaces
@@ -97,11 +126,12 @@ def test_access_to_the_file_with_spaces(tmp_dir_path, create_app_and_client,
     with open(my_file_path, 'w') as fw:
         fw.write(data)
 
-    app, client = yield from create_app_and_client()
+    app = web.Application(loop=loop)
 
     url = os.path.join('/', dir_name, filename)
 
     app.router.add_static('/', tmp_dir_path)
+    client = yield from test_client(app)
 
     r = yield from client.get(url)
     assert r.status == 200
@@ -110,16 +140,17 @@ def test_access_to_the_file_with_spaces(tmp_dir_path, create_app_and_client,
 
 
 @asyncio.coroutine
-def test_access_non_existing_resource(tmp_dir_path, create_app_and_client):
+def test_access_non_existing_resource(tmp_dir_path, loop, test_client):
     """
     Tests accessing non-existing resource
     Try to access a non-exiting resource and make sure that 404 HTTP status
     returned.
     """
-    app, client = yield from create_app_and_client()
+    app = web.Application(loop=loop)
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=True)
+    client = yield from test_client(app)
 
     # Request the root of the static directory.
     r = yield from client.get('/non_existing_resource')
@@ -127,8 +158,30 @@ def test_access_non_existing_resource(tmp_dir_path, create_app_and_client):
     yield from r.release()
 
 
+@pytest.mark.parametrize('registered_path,request_url', [
+    ('/a:b', '/a:b'),
+    ('/a@b', '/a@b'),
+    ('/a:b', '/a%3Ab'),
+])
 @asyncio.coroutine
-def test_unauthorized_folder_access(tmp_dir_path, create_app_and_client):
+def test_url_escaping(loop, test_client, registered_path, request_url):
+    """
+    Tests accessing a resource with
+    """
+    app = web.Application(loop=loop)
+
+    def handler(_):
+        return web.Response()
+    app.router.add_get(registered_path, handler)
+    client = yield from test_client(app)
+
+    r = yield from client.get(request_url)
+    assert r.status == 200
+    yield from r.release()
+
+
+@asyncio.coroutine
+def test_unauthorized_folder_access(tmp_dir_path, loop, test_client):
     """
     Tests the unauthorized access to a folder of static file server.
     Try to list a folder content of static file server when server does not
@@ -137,7 +190,7 @@ def test_unauthorized_folder_access(tmp_dir_path, create_app_and_client):
     my_dir_path = os.path.join(tmp_dir_path, 'my_dir')
     os.mkdir(my_dir_path)
 
-    app, client = yield from create_app_and_client()
+    app = web.Application(loop=loop)
 
     with mock.patch('pathlib.Path.__new__') as path_constructor:
         path = MagicMock()
@@ -148,6 +201,7 @@ def test_unauthorized_folder_access(tmp_dir_path, create_app_and_client):
 
         # Register global static route:
         app.router.add_static('/', tmp_dir_path, show_index=True)
+        client = yield from test_client(app)
 
         # Request the root of the static directory.
         r = yield from client.get('/my_dir')
@@ -157,17 +211,18 @@ def test_unauthorized_folder_access(tmp_dir_path, create_app_and_client):
 
 
 @asyncio.coroutine
-def test_access_symlink_loop(tmp_dir_path, create_app_and_client):
+def test_access_symlink_loop(tmp_dir_path, loop, test_client):
     """
     Tests the access to a looped symlink, which could not be resolved.
     """
     my_dir_path = os.path.join(tmp_dir_path, 'my_symlink')
     os.symlink(my_dir_path, my_dir_path)
 
-    app, client = yield from create_app_and_client()
+    app = web.Application(loop=loop)
 
     # Register global static route:
     app.router.add_static('/', tmp_dir_path, show_index=True)
+    client = yield from test_client(app)
 
     # Request the root of the static directory.
     r = yield from client.get('/my_symlink')
@@ -177,13 +232,13 @@ def test_access_symlink_loop(tmp_dir_path, create_app_and_client):
 
 
 @asyncio.coroutine
-def test_access_special_resource(tmp_dir_path, create_app_and_client):
+def test_access_special_resource(tmp_dir_path, loop, test_client):
     """
     Tests the access to a resource that is neither a file nor a directory.
     Checks that if a special resource is accessed (f.e. named pipe or UNIX
     domain socket) then 404 HTTP status returned.
     """
-    app, client = yield from create_app_and_client()
+    app = web.Application(loop=loop)
 
     with mock.patch('pathlib.Path.__new__') as path_constructor:
         special = MagicMock()
@@ -200,6 +255,7 @@ def test_access_special_resource(tmp_dir_path, create_app_and_client):
 
         # Register global static route:
         app.router.add_static('/', tmp_dir_path, show_index=True)
+        client = yield from test_client(app)
 
         # Request the root of the static directory.
         r = yield from client.get('/special')
@@ -209,14 +265,15 @@ def test_access_special_resource(tmp_dir_path, create_app_and_client):
 
 
 @asyncio.coroutine
-def test_partialy_applied_handler(create_app_and_client):
-    app, client = yield from create_app_and_client()
+def test_partialy_applied_handler(loop, test_client):
+    app = web.Application(loop=loop)
 
     @asyncio.coroutine
     def handler(data, request):
-        return aiohttp.web.Response(body=data)
+        return web.Response(body=data)
 
     app.router.add_route('GET', '/', functools.partial(handler, b'hello'))
+    client = yield from test_client(app)
 
     r = yield from client.get('/')
     data = (yield from r.read())
@@ -225,29 +282,31 @@ def test_partialy_applied_handler(create_app_and_client):
 
 
 def test_system_route():
-    route = SystemRoute(HTTPCreated(reason='test'))
-    assert route.match('any') is None
+    route = SystemRoute(web.HTTPCreated(reason='test'))
     with pytest.raises(RuntimeError):
         route.url()
+    with pytest.raises(RuntimeError):
+        route.url_for()
+    assert route.name is None
+    assert route.resource is None
     assert "<SystemRoute 201: test>" == repr(route)
     assert 201 == route.status
     assert 'test' == route.reason
 
 
 @asyncio.coroutine
-def test_register_route():
-    @asyncio.coroutine
-    def handler(request):
-        return Response()
+def test_412_is_returned(loop, test_client):
 
-    route = PlainRoute('GET', handler, 'test', '/handler/to/path')
-    router = UrlDispatcher()
-    router.register_route(route)
+    class MyRouter(abc.AbstractRouter):
 
-    req = make_mocked_request('GET', '/handler/to/path')
-    info = yield from router.resolve(req)
-    assert info is not None
-    assert 0 == len(info)
-    assert route is info.route
-    assert handler is info.handler
-    assert info.route.name == 'test'
+        @asyncio.coroutine
+        def resolve(self, request):
+            raise web.HTTPPreconditionFailed()
+
+    app = web.Application(router=MyRouter(), loop=loop)
+
+    client = yield from test_client(app)
+
+    resp = yield from client.get('/')
+
+    assert resp.status == 412
